@@ -1,12 +1,52 @@
 """ Common functions towards mnemonic API """
 
-from logging import debug, error
+from logging import debug
 from typing import Any, Dict, Generator, Optional, Text
 
 import requests
 
 from . import worker
 
+# Mapping of message templates provided in 402/503 errors from backend to
+# Exceptions that will be raised
+
+STATUS_CODE_TEMPLATES = {
+    402: {
+        "resource.limit.exceeded": lambda msg: worker.ResourceLimitExceeded(
+            "{message}".format(**msg)),
+    },
+
+    503: {
+        "service.timeout": lambda msg: worker.ServiceTimeout(
+            "{message} ({field}={parameter})".format(**msg)),
+    }
+}
+
+
+def status_code_handler(req: requests.models.Response, res: Dict) -> None:
+    """
+
+    Status Code Handler for non-200 status codes
+    Raise exceptions on combinations of status code and message
+
+    Arguments:
+
+    req: requests Response object
+    res: requests Response content json parsed
+
+    """
+
+    if req.status_code in (402, 503):
+        # Example output on Timeout:
+        # {"responseCode":503,"limit":0,"offset":0,"count":0,"metaData":{},"messages":[{"message":null,"messageTemplate":null,"type":"ACTION_ERROR","field":null,"parameter":null,"timestamp":1606137003885},{"message":"Request timed out, service may be overloaded or unavailable","messageTemplate":"service.timeout","type":"ACTION_ERROR","field":null,"parameter":null,"timestamp":1606137003993}],"data":null,"size":0}
+
+        for msg in res["messages"]:
+            msg_template = msg.get("messageTemplate")
+            if msg_template in STATUS_CODE_TEMPLATES.get(req.status_code, {}):
+                raise STATUS_CODE_TEMPLATES[req.status_code][msg_template](msg)
+
+    raise worker.UnknownResult("Unknown error: {}, {}".format(req, req.content))
+
 
 def batch_query(
         method: Text,
@@ -56,14 +96,8 @@ def batch_query(
         except ValueError:
             raise worker.UnknownResult("Illegal JSON, {}, {}".format(req, req.content))
 
-        if req.status_code == 402:
-            if any([msg.get("message") == "Resource limit exceeded" for msg in res.get("messages")]):
-                error("Resource limit exceeded: {}".format(res))
-                return
-            raise worker.UnknownResult("Unknown 402: {}".format(req.content))
-
-        if not req.status_code == 200:
-            raise worker.UnknownResult("Unknown error: {}, {}".format(req, req.content))
+        if req.status_code != 200:
+            status_code_handler(req, res)
 
         data = res["data"]
         count = res.get("count", 0)
